@@ -172,6 +172,9 @@ classicthemerestorerjso.ctr = {
 	document.getElementById('ctraddon_hctpinfotab').style.visibility = 'collapse';
 	document.getElementById('ctraddon_hctpinfoab').style.visibility = 'collapse';
 	
+	// 'Tabs on bottom' add-ons
+	document.getElementById('ctraddon_tobinfotab').style.visibility = 'collapse';
+	
 	// extra checks to not enable tab width settings while 'TabMixPlus' or 'TabUtilities' is enabled
 	AddonManager.getAddonByID('{dc572301-7619-498c-a57d-39143191b318}', function(addon) {
 	  if(addon && addon.isActive) {
@@ -214,7 +217,7 @@ classicthemerestorerjso.ctr = {
 	  }
 	});
 	
-	// Status4Evar, The Puzzle Piece, Puzzle Toolbars and The Addon Bar Restored
+	// 'Status4Evar', 'Puzzle Bars' and 'The Addon Bar Restored'
 	// override CTRs mov. status bar panel, so CTRs option gets disabled 
 	document.getElementById('ctraddon_pw_statusbar_s4e_info').style.visibility = 'collapse';
 	document.getElementById('ctraddon_pw_statusbar_tpp_info').style.visibility = 'collapse';
@@ -317,6 +320,46 @@ classicthemerestorerjso.ctr = {
 	   }
 	};
 	AddonManager.addAddonListener(HCTPListener);
+	
+	// 'Tabs on bottom' add-on extra info
+	AddonManager.getAddonByID('tabsonbottom@piro.sakura.ne.jp', function(addon) {
+	  if(addon && addon.isActive) {
+		document.getElementById('ctraddon_tobinfotab').style.visibility = 'visible';
+	  }
+	});
+	var TOB1Listener = {
+	   onEnabled: function(addon) {
+		  if(addon.id == 'tabsonbottom@piro.sakura.ne.jp') {
+			document.getElementById('ctraddon_tobinfotab').style.visibility = 'visible';
+		  }
+	   },
+	   onDisabled: function(addon) {
+		  if(addon.id == 'tabsonbottom@piro.sakura.ne.jp') {
+			document.getElementById('ctraddon_tobinfotab').style.visibility = 'collapse';
+		  }
+	   }
+	};
+	AddonManager.addAddonListener(TOB1Listener);
+	
+	// 'Tabs on bottom (Australis)' add-on extra info
+	AddonManager.getAddonByID('jid1-OesGFwaQGIBASw@jetpack', function(addon) {
+	  if(addon && addon.isActive) {
+		document.getElementById('ctraddon_tobinfotab').style.visibility = 'visible';
+	  }
+	});
+	var TOB2Listener = {
+	   onEnabled: function(addon) {
+		  if(addon.id == 'jid1-OesGFwaQGIBASw@jetpack') {
+			document.getElementById('ctraddon_tobinfotab').style.visibility = 'visible';
+		  }
+	   },
+	   onDisabled: function(addon) {
+		  if(addon.id == 'jid1-OesGFwaQGIBASw@jetpack') {
+			document.getElementById('ctraddon_tobinfotab').style.visibility = 'collapse';
+		  }
+	   }
+	};
+	AddonManager.addAddonListener(TOB2Listener);
 
 	// disable bookmark animation checkbox, if 'star button in urlbar' is used
 	if (this.prefs.getBoolPref('starinurl')) document.getElementById('ctraddon_pw_bmanimation').disabled = true;
@@ -520,6 +563,8 @@ classicthemerestorerjso.ctr = {
 	this.ctrpwUnsortBM(this.prefs.getBoolPref("bmbunsortbm"));
 	this.ctrpwHideUrlStopRel(this.prefs.getBoolPref("hideurelstop"));
 	this.ctrpwExtraUrlbar(this.prefs.getBoolPref("extraurlkeycb"));
+	this.ctrpwSearchPopupSize(this.prefs.getBoolPref("osearch_cwidth"));
+	this.ctrpwAeroColors(this.prefs.getBoolPref("aerocolors"));
 	
 	var closetab_value = this.prefs.getCharPref("closetab");
   
@@ -818,7 +863,32 @@ classicthemerestorerjso.ctr = {
 	}
 	
     document.getElementById('ctraddon_extraurltarget_list').disabled = which;
-	document.getElementById('ctraddon_extraurltarget_list').style.visibility = itemvis;
+  },
+  
+  ctrpwSearchPopupSize: function(which) {
+	var itemvis = 'collapse';
+	
+    if(which==true) {
+	  which=false; itemvis = 'visible';
+	} else {
+	  which=true; itemvis = 'collapse';
+	}
+	
+	document.getElementById('ctraddon_os_spsize_minw').disabled = which;
+	document.getElementById('ctraddon_os_spsize_maxw').disabled = which;
+  },
+  
+  ctrpwAeroColors: function(which) {
+	var itemvis = 'collapse';
+	
+    if(which==true) {
+	  which=false; itemvis = 'visible';
+	} else {
+	  which=true; itemvis = 'collapse';
+	}
+	
+    document.getElementById('ctraddon_pw_aerocolorsg').disabled = which;
+	document.getElementById('ctraddon_pw_aerocolorsg').style.visibility = itemvis;
   },
  
   ctrpwCtrOldSearch: function(which) {
@@ -831,10 +901,18 @@ classicthemerestorerjso.ctr = {
 	}
 	
     document.getElementById('ctraddon_pw_ctroldsearchc').disabled = which;
+	document.getElementById('ctraddon_pw_ctroldsearchr').disabled = which;
     document.getElementById('ctraddon_pw_osearch_dm').disabled = which;
     document.getElementById('ctraddon_pw_osearch_iwidth').disabled = which;
     document.getElementById('ctraddon_pw_osearch_meoit').disabled = which;
-	document.getElementById('ctraddon_pw_osearch_iwidth').disabled = which;
+	document.getElementById('ctraddon_pw_osearch_cwidth').disabled = which;
+	if(this.prefs.getBoolPref("osearch_cwidth") && which == false) {
+	  document.getElementById('ctraddon_os_spsize_minw').disabled = false;
+	  document.getElementById('ctraddon_os_spsize_maxw').disabled = false;
+	} else {
+	  document.getElementById('ctraddon_os_spsize_minw').disabled = true;
+	  document.getElementById('ctraddon_os_spsize_maxw').disabled = true;
+	}
 	
   },
  
@@ -1034,11 +1112,11 @@ classicthemerestorerjso.ctr = {
   currentAboutPrefs: function(which) {
 	
 	if(which=="options_win" || which=="options_win_alt") {
-	  document.getElementById('ctraddon_pw_aboutprefswsize').style.visibility = 'visible';
-	  document.getElementById('ctraddon_pw_altoptionsi').style.visibility = 'visible';
+	  document.getElementById('ctraddon_pw_aboutprefswsizew').disabled = false;
+	  document.getElementById('ctraddon_pw_aboutprefswsizeh').disabled = false;
 	} else {
-	  document.getElementById('ctraddon_pw_aboutprefswsize').style.visibility = 'collapse';
-	  document.getElementById('ctraddon_pw_altoptionsi').style.visibility = 'collapse';
+	  document.getElementById('ctraddon_pw_aboutprefswsizew').disabled = true;
+	  document.getElementById('ctraddon_pw_aboutprefswsizeh').disabled = true;
 	}
   },
   
@@ -1114,6 +1192,8 @@ classicthemerestorerjso.ctr = {
 	if(this.appversion >= 43) { 
 	  this.prefs.setBoolPref("oldsearch",true);
 	}
+	if(this.appversion >= 48)
+	  this.prefs.setBoolPref("altautocompl",true);
 	
 	setTimeout(function(){
 		Services.prefs.getBranch("extensions.classicthemerestorer.").setBoolPref("starinurl",true);
@@ -1165,326 +1245,284 @@ classicthemerestorerjso.ctr = {
 	} catch(e) {}
   },
 
-  /* export CTR settings Text */
-  exportCTRpreferences: function() {
-	  
-	  
-	var preflist = Services.prefs.getChildList("extensions.classicthemerestorer.");
-
-	var preferenceArray = [];
-		 
-	// Add filter header
-	preferenceArray.push("CTR_Preferences__DO_NOT_EDIT__'='->booleans__':'->strings__'~'->integers");	
-
-	// Filter preference type and return its value.
-	function prefValue(pref){
-
-	  switch (Services.prefs.getPrefType(pref)){
-		case 32:	return Services.prefs.getCharPref(pref);	break;
-		case 64:	return Services.prefs.getIntPref(pref);		break;
-		case 128:	return Services.prefs.getBoolPref(pref);	break;	
-	  }
-
-	}	
-	
-	//Filter preference type and return its filter value.	
-	function prefType(pref){
-
-	  switch (Services.prefs.getPrefType(pref)){
-		case 32:	return ":";	break;
-		case 64:	return "~";	break;
-		case 128:	return "=";	break;	
-	  }
-
-	}
-
-	for (var i=0; i < preflist.length; i++) {
-
-	  try {
-		// Run Blacklist filter. Exclude all preferences we don't want to export/import.
-		var index = preflist.indexOf(this.blacklist[i]);
-
-		if (index > -1) {
-		  preflist.splice(index, 1);
-		}
-
-		// Filter extensions.classicthemerestorer.*
-		var sliceNdice = preflist[i].replace("extensions.classicthemerestorer.", "");
-		
-		// Populate array	
-		preferenceArray.push(
-		  sliceNdice+prefType(preflist[i])+prefValue(preflist[i]) 
-		);
-
-	  } catch(e) {
-		// Report errors to console
-		Cu.reportError(e);
-	  }
-
-	}	  
-	  
-	// Use new less bulky export for text.
-	saveToFile(preferenceArray);
-	  
-	function saveToFile(patterns) {
-
-	  var nsIFilePicker = Ci.nsIFilePicker;
-	  var fp = Cc["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
-	  var stream = Cc["@mozilla.org/network/file-output-stream;1"].createInstance(Ci.nsIFileOutputStream);
-
-	  fp.init(window, null, nsIFilePicker.modeSave);
-	  fp.defaultExtension = "txt";
-	  fp.defaultString = "CTRpreferences.txt";
-	  fp.appendFilters(nsIFilePicker.filterText);
-
-	  if (fp.show() != nsIFilePicker.returnCancel) {
-		var file = fp.file;
-		if (!/\.txt$/.test(file.leafName.toLowerCase()))
-		  file.leafName += ".txt";
-		if (file.exists())
-		  file.remove(true);
-		file.create(file.NORMAL_FILE_TYPE, parseInt("0666", 8));
-		stream.init(file, 0x02, 0x200, null);
-
-		for (var i = 0; i < patterns.length ; i++) {
-		  patterns[i]=patterns[i]+"\n";
-		  stream.write(patterns[i], patterns[i].length);
-		}
-		stream.close();
-	  }
-	}
-	  
-	return true;
-  },
-  
-  /* import CTR settings */
-  importCTRpreferences: function() {
- 
-	var stringBundle = Services.strings.createBundle("chrome://classic_theme_restorer/locale/messages.file");
-  
-	var pattern = loadFromFile();
-
-	if (!pattern) return false;
-	   
-	if(pattern[0]!="CTR_Preferences__DO_NOT_EDIT__'='->booleans__':'->strings__'~'->integers") {
-	  alert(stringBundle.GetStringFromName("import.error"));
-	  return false;
-	}
-
-	var i, prefName, prefValue;
-	   
-	for (i=1; i<pattern.length; i++){
-	  var index1 = pattern[i].indexOf("="); // for finding booleans
-	  var index2 = pattern[i].indexOf(":"); // for finding strings
-	  var index3 = pattern[i].indexOf("~"); // for finding integers
-
-	  if (index2 > 0){ // find string
-		 prefName  = pattern[i].substring(0,index2);
-		 prefValue = pattern[i].substring(index2+1,pattern[i].length);
-		 
-		 this.prefs.setCharPref(''+prefName+'',''+prefValue+'');
-	  }
-	  else if (index1 > 0){ // find boolean
-		 prefName  = pattern[i].substring(0,index1);
-		 prefValue = pattern[i].substring(index1+1,pattern[i].length);
-		 
-		 // if prefValue string is "true" -> true, else -> false
-		 this.prefs.setBoolPref(''+prefName+'',(prefValue === 'true'));
-	  }
-	  else if (index3 > 0){ // find integer
-		 prefName  = pattern[i].substring(0,index3);
-		 prefValue = pattern[i].substring(index3+1,pattern[i].length);
-		 
-		 this.prefs.setIntPref(''+prefName+'',prefValue);
-	  }
-	}
-	   
-	function loadFromFile() {
-
-	   var nsIFilePicker = Ci.nsIFilePicker;
-	   var fp = Cc["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
-	   var stream = Cc["@mozilla.org/network/file-input-stream;1"].createInstance(Ci.nsIFileInputStream);
-	   var streamIO = Cc["@mozilla.org/scriptableinputstream;1"].createInstance(Ci.nsIScriptableInputStream);
-
-	   fp.defaultExtension = "txt";
-	   fp.defaultString = "CTRpreferences.txt";
-	   fp.init(window, null, nsIFilePicker.modeOpen);
-	   fp.appendFilters(nsIFilePicker.filterText);
-
-	   if (fp.show() != nsIFilePicker.returnCancel) {
-		  stream.init(fp.file, 0x01, parseInt("0444", 8), null);
-		  streamIO.init(stream);
-		  var input = streamIO.read(stream.available());
-		  streamIO.close();
-		  stream.close();
-
-		  var linebreak = input.match(/(((\n+)|(\r+))+)/m)[1];
-		  return input.split(linebreak);
-	   }
-	   return null;
-	}
-	
-	this.needsBrowserRestart();
-	
-	return true;
-  },
-  
-  /* import CTR settings JSON*/
-  importCTRpreferencesJSON: function() {
- 
-	var stringBundle = Services.strings.createBundle("chrome://classic_theme_restorer/locale/messages.file");
-
-	var parjson = loadFromFile();
-
-	if (!parjson) return false;
-	
-	function setPrefValue(pref, val){
-
-	  switch (Services.prefs.getPrefType(pref)){
-		case 32:	return Services.prefs.setCharPref(pref, val);	break;
-		case 64:	return Services.prefs.setIntPref(pref, val);	break;
-		case 128:	return Services.prefs.setBoolPref(pref, val);	break;	
-	  }
-
-	}
-			
-	for (var i=0; i<parjson.length; i++) {					  
-	  try {
-
-		if(parjson[i].preference.match(/extensions.classicthemerestorer./g)){
-			//To import previously generated preference export.
-			setPrefValue(parjson[i].preference, parjson[i].value);
-		} else{
-			setPrefValue('extensions.classicthemerestorer.' + parjson[i].preference, parjson[i].value);
-		}
-
-	  } catch(e) {
-		// Report errors to console
-		Cu.reportError(e);
-	  }
-	}	
-
 	// Need to check if json is valid. If json not valid. don't continue and show error.
-	function IsJsonValid(text) {
+	IsJsonValid: function(aData) {
+        try {
+            JSON.parse(aData);
+        } catch (e) {
+            return false;
+        }
+        return true;
+    },
 
-	  try { JSON.parse(text); }
-	  catch (e) { return false; }
-	  return true;
+	/* Export CTR preferences Text|Json */
+    ExportPreferences: function(aPattern) {
+			
+			if (!aPattern == "txt" || !aPattern == "json") return false;
+		
+        var preferenceList = Services.prefs.getChildList("extensions.classicthemerestorer.");
+        var preferenceArray = null;
 
-	}				
-	 
-	function loadFromFile() {
+        if (aPattern == "txt") {
+            preferenceArray = [];
+            // Add filter header.
+            preferenceArray.push("CTR_Preferences__DO_NOT_EDIT__'='->booleans__':'->strings__'~'->integers");
+        }
 
-	   var nsIFilePicker = Ci.nsIFilePicker;
-	   var fp = Cc["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
-	   var stream = Cc["@mozilla.org/network/file-input-stream;1"].createInstance(Ci.nsIFileInputStream);
-	   var streamIO = Cc["@mozilla.org/scriptableinputstream;1"].createInstance(Ci.nsIScriptableInputStream);
+        if (aPattern == "json") {
+            preferenceArray = {
+                preference: [],
+                value: []
+            };
+        }
+        // Filter preference type and return its value.
+        function _prefValue(pref) {
+            switch (Services.prefs.getPrefType(pref)) {
+                case 32:return Services.prefs.getCharPref(pref);break;
+                case 64:return Services.prefs.getIntPref(pref);break;
+                case 128:return Services.prefs.getBoolPref(pref);break;
+            }
+        }
 
-	   fp.defaultExtension = "json";
-	   fp.defaultString = "CTRpreferences.json";
-	   fp.init(window, null, nsIFilePicker.modeOpen);
-	   fp.appendFilters(nsIFilePicker.filterAll);
+        // Filter preference type and return its filter value.	
+        function _prefType(pref) {
+            switch (Services.prefs.getPrefType(pref)) {
+                case 32:return ":";break;
+                case 64:return "~";break;
+                case 128:return "=";break;
+            }
+        }
 
-	   if (fp.show() != nsIFilePicker.returnCancel) {
-		  stream.init(fp.file, 0x01, parseInt("0444", 8), null);
-		  streamIO.init(stream);
-		  var input = streamIO.read(stream.available());
-		  streamIO.close();
-		  stream.close();
 
-		 var text = input;
 
-		  if(!IsJsonValid(text)){
-			  alert(stringBundle.GetStringFromName("import.error"));
-			  return false;
-		  } else{
-			return JSON.parse(input);
-		  }
-	   }
-	   return null;
-	}
+        for (var i = 0; i < preferenceList.length; i++) {
+            try {
+                // Run Blacklist filter. Exclude all preferences we don't want to export/import.
+                var index = preferenceList.indexOf(this.blacklist[i]);
+
+                if (index > -1) {
+                    preferenceList.splice(index, 1);
+                }
+
+                if (aPattern == "txt") {
+                    // Filter extensions.classicthemerestorer.*
+                    var sliceNdice = preferenceList[i].replace("extensions.classicthemerestorer.", "");
+
+                    // Populate array	
+                    preferenceArray.push(
+                        sliceNdice + _prefType(preferenceList[i]) + _prefValue(preferenceList[i])
+                    );
+                }
+
+                if (aPattern == "json") {
+					// Populate array
+                    preferenceArray.preference.push({
+                        "preference": preferenceList[i].replace("extensions.classicthemerestorer.", ""),
+                        "value": _prefValue(preferenceList[i])
+                    });
+                }
+
+            } catch (e) {
+                // Report errors to console
+                Cu.reportError(e);
+            }
+        }
+        // Use new less bulky export for text.
+        classicthemerestorerjso.ctr.saveToFile(preferenceArray, aPattern);
+        // Clear preferenceArray after export.
+        preferenceArray = [];
+        return true;
+    },
 	
-	this.needsBrowserRestart();
-	
-	return true;
-  },
-  
-  /* export CTR settings JSON */
-  exportCTRpreferencesJSON: function() {
-
-	var preflist = Services.prefs.getChildList("extensions.classicthemerestorer.");
-
-	var preferenceArray = {
-	  preference: [],
-	  value: []
-	};
-
-
-	function prefValue(pref){
-
-	  switch (Services.prefs.getPrefType(pref)){
-		case 32:	return Services.prefs.getCharPref(pref);	break;
-		case 64:	return Services.prefs.getIntPref(pref);		break;
-		case 128:	return Services.prefs.getBoolPref(pref);	break;	
-	  }
-
-	}
-
-	for (var i=0; i < preflist.length; i++) {
-
-	  try {
-		// 'Blacklist' filter. Exclude all preferences we don't want to export/import.
-		var index = preflist.indexOf(this.blacklist[i]);
-
-		if (index > -1) {
-		  preflist.splice(index, 1);
+	/* Import CTR preferences Text|Json */
+	ImportPreferences: function(aPattern) {
+		
+		if (!aPattern == "txt" || !aPattern == "json") return false;
+		
+		var stringBundle = Services.strings.createBundle("chrome://classic_theme_restorer/locale/messages.file");
+		var pattern = null;
+		
+		if (aPattern == "txt") {
+			pattern = classicthemerestorerjso.ctr.loadFromFile("txt");
+		}
+		
+		if (aPattern == "json") {
+			pattern = classicthemerestorerjso.ctr.loadFromFile("json");
+		}		
+		
+		if (!pattern) return false;
+		   
+		if(pattern[0]!="CTR_Preferences__DO_NOT_EDIT__'='->booleans__':'->strings__'~'->integers" && aPattern == "txt") {
+		  alert(stringBundle.GetStringFromName("import.error"));
+		  return false;
 		}
 
-		preferenceArray.preference.push({
-		  "preference" : preflist[i].replace("extensions.classicthemerestorer.", ""),
-		  "value" : prefValue(preflist[i])
-		});
+		function _setPrefValue(pref, val){
 
-	  } catch(e) {
-		// Report errors to console
-		Cu.reportError(e);
-	  }
+		  switch (Services.prefs.getPrefType(pref)){
+			case 32:	return Services.prefs.setCharPref(pref, val);	break;
+			case 64:	return Services.prefs.setIntPref(pref, val);	break;
+			case 128:	return Services.prefs.setBoolPref(pref, val);	break;	
+		  }
 
-	}
 
-	saveToFile(preferenceArray);
-	  
-	function saveToFile(patterns) {
+		}
+		
+		if (aPattern == "txt") {
+			var i, prefName, prefValue;
+			   
+			for (i=1; i<pattern.length; i++){
+			  var index1 = pattern[i].indexOf("="); // for finding booleans
+			  var index2 = pattern[i].indexOf(":"); // for finding strings
+			  var index3 = pattern[i].indexOf("~"); // for finding integers
 
-	  var nsIFilePicker = Ci.nsIFilePicker;
-	  var fp = Cc["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
-	  var stream = Cc["@mozilla.org/network/file-output-stream;1"].createInstance(Ci.nsIFileOutputStream);
+			  if (index2 > 0){ // find string
+				 prefName  = pattern[i].substring(0,index2);
+				 prefValue = pattern[i].substring(index2+1,pattern[i].length);
+				 
+				 this.prefs.setCharPref(''+prefName+'',''+prefValue+'');
+			  }
+			  else if (index1 > 0){ // find boolean
+				 prefName  = pattern[i].substring(0,index1);
+				 prefValue = pattern[i].substring(index1+1,pattern[i].length);
+				 
+				 // if prefValue string is "true" -> true, else -> false
+				 this.prefs.setBoolPref(''+prefName+'',(prefValue === 'true'));
+			  }
+			  else if (index3 > 0){ // find integer
+				 prefName  = pattern[i].substring(0,index3);
+				 prefValue = pattern[i].substring(index3+1,pattern[i].length);
+				 
+				 this.prefs.setIntPref(''+prefName+'',prefValue);
+			  }
+			}
+		}
+		
+		if (aPattern == "json") {
+			for (var i=0; i<pattern.length; i++) {					  
+			  try {
 
-	  fp.init(window, null, nsIFilePicker.modeSave);
-	  fp.defaultExtension = "json";
-	  fp.defaultString = "CTRpreferences.json";
-	  fp.appendFilters(nsIFilePicker.filterAll);
+				if(pattern[i].preference.match(/extensions.classicthemerestorer./g)){
+					// To import previously generated preference export.
+					_setPrefValue(pattern[i].preference, pattern[i].value);
+				} else{
+					_setPrefValue('extensions.classicthemerestorer.' + pattern[i].preference, pattern[i].value);
+				}
 
-	  if (fp.show() != nsIFilePicker.returnCancel) {
-		var file = fp.file;
-		if (!/\.json$/.test(file.leafName.toLowerCase()))
-		  file.leafName += ".json";
-		if (file.exists())
-		  file.remove(true);
-		file.create(file.NORMAL_FILE_TYPE, parseInt("0666", 8));
-		stream.init(file, 0x02, 0x200, null);
+			  } catch(e) {
+				// Report errors to console
+				Cu.reportError(e);
+			  }
+			}
+		}
+		
+		this.needsBrowserRestart();		
+		return true;
+	},
+	
+	saveToFile: function(aPattern, aType) {
+		try{
+			
+			if (!aType === "txt" || !aType === "json" || aPattern.length === 0) return false;
 
-		var patternItems = JSON.stringify(patterns.preference);
+			const nsIFilePicker = Ci.nsIFilePicker;
+			var fp = Cc["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
+			var stream = Cc["@mozilla.org/network/file-output-stream;1"].createInstance(Ci.nsIFileOutputStream);
 
-		stream.write(patternItems, patternItems.length)
+			fp.init(window, null, nsIFilePicker.modeSave);
+			fp.defaultExtension = aType;
+			fp.defaultString = "CTRpreferences." + aType;
 
-		stream.close();
-	  }
-	}
+			if (aType === "txt") {
+			  fp.appendFilters(nsIFilePicker.filterText);
+			} else if (aType === "json") {
+			  fp.appendFilters(nsIFilePicker.filterAll);
+			}
 
-	return true;
+			if (fp.show() != nsIFilePicker.returnCancel) {
+			  var file = fp.file;
+			  if (aType === "txt") {
+				if (!/\.txt$/.test(file.leafName.toLowerCase()))
+				  file.leafName += ".txt";
+			  } else if (aType === "json") {
+				if (!/\.json$/.test(file.leafName.toLowerCase()))
+				  file.leafName += ".json";
+			  }
+			  if (file.exists())
+				file.remove(true);
+			  file.create(file.NORMAL_FILE_TYPE, parseInt("0666", 8));
+			  stream.init(file, 0x02, 0x200, null);
 
-  }, 
+			  switch (aType) {
+				case "txt":
+				  for (var i = 0; i < aPattern.length; i++) {
+					aPattern[i] = aPattern[i] + "\n";
+					stream.write(aPattern[i], aPattern[i].length);
+				  }
+				  break;
+				case "json":
+				  var patternItems = JSON.stringify(aPattern.preference);
+				  stream.write(patternItems, patternItems.length)
+				  break;
+			  }
+			  stream.close();
+			}
+			return true;
+		  } catch(e) {
+			Cu.reportError(e);
+		  }
+	  },
+
+	  loadFromFile: function(aType) {
+		  try{
+			if (aType === "txt" || aType === "json") {} else {
+			  return false;
+			}
+			
+			var stringBundle = Services.strings.createBundle("chrome://classic_theme_restorer/locale/messages.file");
+			const nsIFilePicker = Ci.nsIFilePicker;
+			var fp = Cc["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
+			var stream = Cc["@mozilla.org/network/file-input-stream;1"].createInstance(Ci.nsIFileInputStream);
+			var streamIO = Cc["@mozilla.org/scriptableinputstream;1"].createInstance(Ci.nsIScriptableInputStream);
+
+			fp.defaultExtension = aType;
+			fp.defaultString = "CTRpreferences." + aType;
+			fp.init(window, null, nsIFilePicker.modeOpen);
+			if (aType === "txt") {
+			  fp.appendFilters(nsIFilePicker.filterText);
+			} else if (aType === "json") {
+			  fp.appendFilters(nsIFilePicker.filterAll);
+			}
+
+			if (fp.show() != nsIFilePicker.returnCancel) {
+			  stream.init(fp.file, 0x01, parseInt("0444", 8), null);
+			  streamIO.init(stream);
+			  var input = streamIO.read(stream.available());
+			  streamIO.close();
+			  stream.close();
+
+			  switch (aType) {
+				case "txt":
+				  var linebreak = input.match(/(((\n+)|(\r+))+)/m)[1];
+				  return input.split(linebreak);
+				  break;
+				case "json":
+				  var text = input;
+				  if (!classicthemerestorerjso.ctr.IsJsonValid(text)) {
+					alert(stringBundle.GetStringFromName("import.error"));
+					return false;
+				  } else {
+					return JSON.parse(input);
+				  }
+				  break;
+			  }
+
+			}
+			return null;
+		  } catch(e) {
+			Cu.reportError(e);
+		  }
+	  },
  
   onCtrPanelSelect: function() {
     var ctrAddonPrefBoxTab = document.getElementById("CtrRadioGroup");
